/***************************************************************************
                          dierollerapp.cpp  -  description
                             -------------------
    begin                : Tue Apr  1 13:59:16 CST 2003
    copyright            : (C) 2003 by Ralph M. Churchill
    email                : churchillrm@cox.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dierollerapp.h"

#include <qvariant.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

#include <kstddirs.h>

float convertToDie(const QString& dieTypeStr)
{
    // all dice are listed with a "d" prepended, simply strip that and return as a float
    return dieTypeStr.right(dieTypeStr.length()-1).toFloat();
}

DieRollerApp::DieRollerApp(QWidget *parent, const char *name) : DieRoller(parent, name)
{
  init();
}

DieRollerApp::~DieRollerApp()
{
}

void DieRollerApp::init()
{
    noDiceLineEdit->setText("1");
    modLineEdit->setText("0");
    // resultLCDNumber->setBackgroundColor(darkBlue);
    resultTextLabel->setText("Result (1d20)");
    selectedDie = 20;

    // initialize all images from kdedir()/share/apps/dierollerapp/pics
    d4PushButton->setPixmap(locate("data","dierollerapp/pics/d4.png"));
    d6PushButton->setPixmap(locate("data","dierollerapp/pics/d6.png"));
    d8PushButton->setPixmap(locate("data","dierollerapp/pics/d8.png"));
    d10PushButton->setPixmap(locate("data","dierollerapp/pics/d10.png"));
    d12PushButton->setPixmap(locate("data","dierollerapp/pics/d12.png"));
    d20PushButton->setPixmap(locate("data","dierollerapp/pics/d20.png"));
   
}

void DieRollerApp::roll()
{
    int noDice = noDiceLineEdit->text().toLong();
    if(noDice < 1) return; // can't roll less than one die...

    float dieType = selectedDie;
    int mod = modLineEdit->text().toLong();

    // Instead of rolling one die and multiplying it by mod, roll one die
    // mod times to simulate actual rolling of multiple dice
    int totalRandNum = 0;
    for(int i=0; i<noDice; i++) {
	    int randNum =1+(int) (dieType*rand()/(RAND_MAX+1.0));

	    if (dieType == 20.0) {
	      if(randNum == 20){
		      // Natural 20!
		      if(nat20CheckBox->isChecked()){
		        QMessageBox::information(this, "Die Roller",
					     "Congratulations, you rolled a Natural 20!");
		      }
	      }
	      else if(randNum == 1){
		      // Natural 1!
		      if(nat1CheckBox->isChecked()){
		        QMessageBox::information(this, "Die Roller",
					     "Sorry, you rolled a Natural 1!");
		      }
	      }
	    }

      totalRandNum += randNum;
    }

    int result = totalRandNum + mod;

    QString roll = QString::number(totalRandNum);
    roll += (mod>=0)?" + ":" - ";
    roll += QString::number(abs(mod));
    roll += " = " + QString::number(result);

    resultLineEdit->setText(roll);
    resultLCDNumber->display(result);
}

void DieRollerApp::reset()
{
    noDiceLineEdit->setText("1");
    noDiceLineEdit->setFocus();
    modLineEdit->setText("0");
    resultLCDNumber->display("0");
    d20PushButton->setOn(true);
    resultLineEdit->setText("");
}

void DieRollerApp::updateResultLabel()
{
    QString dieRoll = "Result (" + noDiceLineEdit->text() +
		      "d" + QString::number(selectedDie) + ")";
    resultTextLabel->setText(dieRoll);
}

void DieRollerApp::d4selected()
{
    selectedDie = 4;
}

void DieRollerApp::d6selected()
{
    selectedDie = 6;
}

void DieRollerApp::d20selected()
{
    selectedDie = 20;
}

void DieRollerApp::d8selected()
{
    selectedDie = 8;
}

void DieRollerApp::d10selected()
{
    selectedDie = 10;
}

void DieRollerApp::d12selected()
{
    selectedDie = 12;
}
